# YML CBuild Format
<!-- markdownlint-disable MD013 -->
<!-- markdownlint-disable MD036 -->

The following chapter explains the YML CBuild format that describes how to build a complete application.

## Table of Contents

- [YML CBuild Format](#yml-cbuild-format)
  - [Table of Contents](#table-of-contents)
  - [CBuild Output Files](#cbuild-output-files)
    - [Directory Structure](#directory-structure)
    - [File Structure of `*.cbuild-idx.yml`](#file-structure-of-cbuild-idxyml)
    - [File Structure of `*.cbuild.yml`](#file-structure-of-cbuildyml)
  - [CBuild-specific Nodes](#cbuild-specific-nodes)
    - [`licenses:`](#licenses)
    - [`csolution:`](#csolution)
    - [`contexts:`](#contexts)
    - [`output-dirs:`](#output-dirs)
    - [`packs:`](#packs)
  - [Source File Management](#source-file-management)
    - [`groups:`](#groups)
    - [`files:`](#files)
    - [`components:`](#components)
    - [`component-files:`](#component-files)
  - [`constructed-files:`](#constructed-files)
    - [`messages:`](#messages)

## CBuild Output Files

The `*.cbuild-idx.yml` and `*.cbuild.yml` output files are generated by the `csolution` utility.

File                | Description
:-------------------|:----------------------
`*.cbuild-idx.yml`  | Index file of all `*.cbuild.yml` build descriptions; contains also overall information for the application.
`*.cbuild.yml`      | Build description of a single `*.cproject.yml` file.

The `*.cbuild.yml` output file has the following usage:

- It contains all information for the build step of a project that is part of an application. - As it contains information about all software packs used including version information, this file can be used as `lock-file` to ensure that subsequent runs of `csolution` use the very same software packs.
- It can be used as input file for a generator as it contains explicit information about source files and avoids the complexity of a pack data management at the generator level.

### Directory Structure

As `csolution` based projects should be portable across different host computers, **relative file references** are used within the directory structure.

- All file references use relative paths to the base directory of the related `*.yml` file.  Files that are within the file structure of the `csolution` base directory are also referenced using relative paths, i.e. `../layers/layer1/source-file1.c`.

- Files that are located in the [CMSIS-Pack root directory](https://github.com/Open-CMSIS-Pack/cmsis-toolbox/blob/main/docs/installation.md#environment-variables) are prefixed with %CMSIS_PACK_ROOT%.

- Files outside of the directory structure of `csolution` based application use absolute paths. If absolute paths are used, a `warning` is issued in the `*.cbuild-idx.yml` file.

A typical directory structure of a `csolution` based application that uses common layers source files is shown below.

```yml
📦 # csolution base directory
 ┣ myapp.csolution.yml
 ┣ myapp.cbuild-idx.yml
 ┣ 📂 project1
 ┃  ┣  mypro1.cproject.yml
 ┃  ┣  mypro1.cbuild.yml       # file references are relative to directory project1
 ┣ 📂 project2
 ┃  ┣  mypro2.cproject.yml
 ┃  ┣  mypro2.cbuild.yml       # file references are relative to directory project2
 ┣ 📂 layer
```

### File Structure of `*.cbuild-idx.yml`

`build-idx:`                                       | Content
:--------------------------------------------------|:------------------------------------
&nbsp;&nbsp; [`licenses:`](#licenses)              | List of common licenses used by solution
&nbsp;&nbsp; [`csolution:`](#csolution)            | List of `*.yml` input files used to generate this application.
&nbsp;&nbsp; [`builds:`](#contexts)                | List of project contexts that belong to the solution.

```yml
  licenses: # all licenses used in the projects  
  ...

  builds:
  - cbuild: .\project1\mypro1.cbuild.yml
    contexts:
    - context: .release+AVH
    - context: .debug+STM32U5
      messages:
      - info: contains absolute path specifications
  - cbuild: .\project2\mypro2.cbuild.yml
    
```

### File Structure of `*.cbuild.yml`

The `cbuild.yml` file is structured into several sections.  The top-level structure is outlined below.

`build:`                                           | Content
:--------------------------------------------------|:------------------------------------
&nbsp;&nbsp; [`packs:`](#packs)                    | List of common packs used to generate the solution
&nbsp;&nbsp; [`licenses:`](#licenses)              | List of common packs used to generate the solution
&nbsp;&nbsp; [`csolution:`](#csolution)            | List of `*.yml` input files used to generate this `*.cbuild.yml` file.
&nbsp;&nbsp; [`contexts:`](#contexts)              | List of project contexts that belong to the solution.

**Example:**

ToDo replace with a real output file once a first implementation is available.

```yml
build:
# section that defines global setup for all projects (only packs and licenses are listed here)

  packs:     # all packs used in this projects (exception when a context uses a different pack version)
    - pack: ARM::CMSIS@5.9.0
    - pack: ARM::CMSIS-Driver@2.7.2
    - pack: ARM::CMSIS-FreeRTOS@10.4.6
    - pack: ARM::mbedTLS@1.7.0
    - pack: Keil::STM32U5xx_DFP@2.0.0
    - pack: Keil::B-U585I-IOT02A_BSP@1.0.0

  ...
# project specific section; almost identical with cproject, except all types are explicit, components are expanded 

  contexts:
  - context: MQTT_MutualAuth_Demo.Debug+AVH_MPS4_Corstone-300  

    compiler: AC6@6.18   

    board: B-U585I-IOT02A
    device: SSE-300-MPS3     # assuming that all projects use the same device

    packs:   # packs that are specific to this project context

    add-paths:
    - ./config_files
    - ./amazon-freertos/demos/include
    - ./amazon-freertos/demos/network_manager
  
    defines:
    - CONFIG_CORE_MQTT_MUTUAL_AUTH_DEMO_ENABLED

    components:
    - component: ARM::CMSIS:RTOS2:FreeRTOS&Cortex-M
      from-pack: ARM::CMSIS-FreeRTOS@10.4.6
      selected-by: CMSIS:RTOS2:FreeRTOS
      condition: ARMv6_7_8-M Device 
      component-files:
      - file: %CMSIS_PACK_ROOT%/ARM/CMSIS-FreeRTOS/10.4.6/CMSIS/RTOS2/FreeRTOS/Source/ARM/clib_arm.c
        category: sourceC
      - file: %CMSIS_PACK_ROOT%/ARM/CMSIS-FreeRTOS/10.4.6/CMSIS/RTOS2/FreeRTOS/Source/cmsis_os2.c
        category: sourceC
      - file: %CMSIS_PACK_ROOT%/ARM/CMSIS-FreeRTOS/10.4.6/CMSIS/RTOS2/FreeRTOS/Source/os_systick.c
        category: sourceC
    - component: ARM::RTOS&FreeRTOS:Config&CMSIS RTOS2
      from-pack: ARM::CMSIS-FreeRTOS@10.4.6
      selected-by: RTOS&FreeRTOS:Config&CMSIS RTOS2
      component-files:
      - file: ./RTE/RTOS/FreeRTOSConfig.h
        category: include
        attr: config

    - component: ARM::Security:mbed TLS
      from-pack: ARM::mbedTLS@1.7.0
      selected-by: Security:mbed TLS

      defines:
      - MBEDTLS_CONFIG_FILE="aws_mbedtls_config.h"
      component-files:
      ...

    groups:
    - group: Documentation
      files: 
      - file: ./README.md
    - group: main
      files:
      - file: ./app_main.c
      - file: ./tfm_ns_iterface_freertos.c

    messages:  # error/warning/info messages that apply to this project context
    ...  

    constructed-files:
      - file: ./RTE/RTE_Components.h
      - file: ./RTE/RTE_PreInclude.h

  - context: # next project context
```

## CBuild-specific Nodes

### `licenses:`

The `licenses:` node collects the software licenses from all components and software packs that are used in a `csolution` based software project.

`licenses:`                                                        | Content
:------------------------------------------------------------------|:------------------------------------
`- license:`                                                       | SPDX-name of the license, `<proprietary>` to indicate a dedicated license agreement, or `<unknown>`.
&nbsp;&nbsp;`license-agreement:`                                   | In cased of `<proprietary>` a link to the license agreement.
&nbsp;&nbsp; [`packs:`]                                            | List of software packs that use this license.
&nbsp;&nbsp; [`components:`]                                       | List of software components that use this license.

**Example:**

```yml
licenses:
  - license: Apache-2.0
    packs:
    - pack: ARM::CMSIS@5.9.0
    - pack: ARM::mbedTLS@1.7.0
    components:
    - component: ARM::CMSIS:RTOS2:FreeRTOS&Cortex-M

  - license: <unknown>
    packs:
    - pack: ARM::CMSIS-Driver@2.7.2

  - license: <proprietary>
    license-agreement: %CMSIS_PACK_ROOT%/Keil/STM32U5xx_DFP/2.0.0/Package_license.md
    packs:
    - pack: Keil::STM32U5xx_DFP@2.0.0
```

### `csolution:`

The `csolution:` node lists all `*.yml` input files that are used to compose the `*.cbuild.yml`.
It includes `*.cdefault.yml`, `*.csolution.yml`, `*.cproject.yml`, `*.clayer.yml`, and `*.genlayer.yml`
files and represents the file hierarchy of the solution.

`csolution:`                                                       | Content
:------------------------------------------------------------------|:------------------------------------
&nbsp;&nbsp;`cdefault:`                                            | File of the `*.cdefault.yml` file.
&nbsp;&nbsp;`file:`                                                | File of the `*.csolution.yml` file.
&nbsp;&nbsp;`projects:`                                            | List of `*.cproject.yml` files.
&nbsp;&nbsp; `- file:`                                             | List of `*.clayer.yml` files.
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; `layers:`                     | List of layer files.
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; `- file:`                     | File name of the `*.clayer.yml` or `*.genlayer.yml` file:

**Example:**

```yml
csolution:
  cdefault: ./Demo.cdefault.yml
  file: ./Demo.csolution.yml
  projects:
  - file: ./Demo1.cproject.yml
    layers:
    - file: ./Board/IMXRT1050-EVKB/Board.clayer.yml
    - file: ./Board/B-U585I-IOT02A/Board.clayer.yml
    - file: ./Interface/AWS/FreeRTOS+TCP/Interface.clayer.yml
  - file: ./Demo2.cproject.yml
  :
```

### `contexts:`

The `contexts:` node is the start of project context list.

`contexts:`                                                        | Content
:------------------------------------------------------------------|:------------------------------------
`- context:`                                                       | Project context name
&nbsp;&nbsp; `description:`                                        | Brief project description (copied from `cproject.yml`)
&nbsp;&nbsp; [`compiler:`](YML-Input-Format.md#compiler)           | Compiler used for project generation.
&nbsp;&nbsp; [`packs:`](#packs)                                    | List of packs that are specify to this project context.
&nbsp;&nbsp; [`language-C:`](YML-Input-Format.md#language-c)       | Optimize level for code generation.
&nbsp;&nbsp; [`language-CPP:`](YML-Input-Format.md#language-cpp)   | Optimize level for code generation.
&nbsp;&nbsp; `output-dirs:`                                        | Output directories for code generation.
&nbsp;&nbsp; [`output-type:`](YML-Input-Format.md#output-type)     | Generate executable (default) or library.
&nbsp;&nbsp; [`optimize:`](YML-Input-Format.md#optimize)           | Optimize level for code generation.
&nbsp;&nbsp; [`linker:`](YML-Input-Format.md#linker)               | Instructions for the linker.
&nbsp;&nbsp; [`debug:`](YML-Input-Format.md#debug)                 | Generation of debug information.
&nbsp;&nbsp; [`warnings:`](YML-Input-Format.md#warnings)           | Control generation of compiler diagnostics.
&nbsp;&nbsp; [`define:`](YML-Input-Format.md#define)               | Global preprocessor (#define) symbols for code generation.
&nbsp;&nbsp; [`add-path:`](YML-Input-Format.md#add-path)           | Additional include file paths.
&nbsp;&nbsp; [`misc:`](YML-Input-Format.md#misc)                   | Literal tool-specific controls.
&nbsp;&nbsp; [`board:`](YML-Input-Format.md#board)                 | Board specification.
&nbsp;&nbsp; [`device:`](YML-Input-Format.md#device)               | Device specification.
&nbsp;&nbsp; [`processor:`](YML-Input-Format.md#processor)         | Processor specific settings.
&nbsp;&nbsp; [`groups:`](#groups)                                  | List of source file groups along with source files.
&nbsp;&nbsp; [`components:`](#components)                          | List of software components used.
&nbsp;&nbsp; [`messages:`](#messages)                              | List of messages (errors, warnings, etc.) that apply to this project context.
&nbsp;&nbsp; [`constructed-files:`](#constructed-files)            | List of files that are generated by RTE management of `csolution` tool.

> **Note:** Most fields are combined can collect the common settings that apply to any `group:` or `component:`
> in the project.

### `output-dirs:`

`output-dirs:`               | Content
:----------------------------|:------------------------------------
&nbsp;&nbsp; `intdir:`       | Specifies the directory for the interim files.
&nbsp;&nbsp; `outdir:`       | Specifies the directory for the build output files.

### `packs:`

The `packs:` node is the start of a pack list that is used in the `csolution` application.

`packs:`                                              | Content
:-----------------------------------------------------|:------------------------------------
`- pack:`                                             | Explicit pack specification (additive) with exact version information used.
&nbsp;&nbsp; `path:`                                  | For packs that are local to the `csolution`, explicit path name that stores the software pack
&nbsp;&nbsp; `local-repository:`                      | For packs that are managed as local repository, the absolute path name to the working directory of that pack.
&nbsp;&nbsp; `devices:`                               | A list of device definitions defined by this pack and used by the solution.
&nbsp;&nbsp; `boards:`                                | A list of board definitions defined by this pack and used by the solution.

>**NOTE:** Projects that use `local-repository:` might be not portable across systems.

**Example:**

```yml
packs:                                  # start section that specifics software packs
  - pack: Keil::MDK-Middleware@7.13.0 
  - pack: ARM::CMSIS-FreeRTOS@10.4.2
  - pack: NXP::K32L3A60_DFP@10.6.0
    devices:
    - device: K32L3A61
  - pack: MDK-Packs:IoT_Socket@1.3.1
    local-repository: C:/w/IoT_Socket/
```

## Source File Management

Keyword          | Description
:----------------|:------------------------------------
`groups:`        | Start of a list that adds source groups and files.
`components:`    | Start of a list that adds software components.

### `groups:`

`groups:`                                                | Content
:--------------------------------------------------------|:------------------------------------
`- group:`                                               | Name of the group.
&nbsp;&nbsp; [`optimize:`](YML-Input-Format.md#optimize) | Optimize level for code generation.
&nbsp;&nbsp; [`debug:`](YML-Input-Format.md#debug)       | Generation of debug information.
&nbsp;&nbsp; [`warnings:`](YML-Input-Format.md#warnings) | Control generation of compiler diagnostics.
&nbsp;&nbsp; [`define:`](YML-Input-Format.md#define)     | Define symbol settings for code generation.
&nbsp;&nbsp; [`add-path:`](YML-Input-Format.md#add-path) | Additional include file paths.
&nbsp;&nbsp; [`misc:`](YML-Input-Format.md#misc)         | Literal tool-specific controls.
&nbsp;&nbsp; [`groups:`](#groups)                        | Start a nested list of groups.
&nbsp;&nbsp; [`files:`](#files)                          | Start a list of files.

### `files:`

`files:`                                                 | Content
:--------------------------------------------------------|:------------------------------------
`- file:`                                                | Name of the file.
&nbsp;&nbsp; [`optimize:`](YML-Input-Format.md#optimize) | Optimize level for code generation.
&nbsp;&nbsp; [`debug:`](YML-Input-Format.md#debug)       | Generation of debug information.
&nbsp;&nbsp; [`warnings:`](YML-Input-Format.md#warnings) | Control generation of compiler diagnostics.
&nbsp;&nbsp; [`define:`](YML-Input-Format.md#define)     | Define symbol settings for code generation.
&nbsp;&nbsp; [`add-path:`](YML-Input-Format.md#add-path) | Additional include file paths.
&nbsp;&nbsp; [`misc:`](YML-Input-Format.md#misc)         | Literal tool-specific controls.

### `components:`

`components:`                                            | Content
:--------------------------------------------------------|:------------------------------------
`- component:`                                           | Name of the software component.
&nbsp;&nbsp; `from-pack:`                                | Pack that defines this component.
&nbsp;&nbsp; `selected-by:`                              | The original component name used in `cproject/clayer.YML`.
&nbsp;&nbsp; [`optimize:`](YML-Input-Format.md#optimize) | Optimize level for code generation.
&nbsp;&nbsp; [`debug:`](YML-Input-Format.md#debug)       | Generation of debug information.
&nbsp;&nbsp; [`warnings:`](YML-Input-Format.md#warnings) | Control generation of compiler diagnostics.
&nbsp;&nbsp; [`define:`](YML-Input-Format.md#define)     | Define symbol settings for code generation.
&nbsp;&nbsp; [`add-path:`](YML-Input-Format.md#add-path) | Additional include file paths.
&nbsp;&nbsp; [`misc:`](YML-Input-Format.md#misc)         | Literal tool-specific controls.
&nbsp;&nbsp; `instances:`                                | Configure component instances.
&nbsp;&nbsp; [`generator:`]                              | Generator ID for components that are configurable via a generator.
&nbsp;&nbsp; [`component-files:`](#component-files)      | Files that belong to this component.
&nbsp;&nbsp; [`condition:`](https://open-cmsis-pack.github.io/Open-CMSIS-Pack-Spec/main/html/pdsc_conditions_pg.html#element_condition) | Reference to the condition ID of the software pack that triggered inclusion of this component.

### `component-files:`

<!-- markdownlint-capture -->
<!-- markdownlint-disable MD013 -->

`component-files:`                                       | Content
:--------------------------------------------------------|:------------------------------------
`- file:`                                                | Name and path to the file.
&nbsp;&nbsp; [`category:`](https://open-cmsis-pack.github.io/Open-CMSIS-Pack-Spec/main/html/pdsc_components_pg.html#FileCategoryEnum) | File category according Open-CMSIS-Pack specification
&nbsp;&nbsp; [`attr:`](https://open-cmsis-pack.github.io/Open-CMSIS-Pack-Spec/main/html/pdsc_components_pg.html#FileAttributeEnum) | File category according Open-CMSIS-Pack specification
&nbsp;&nbsp; [`condition:`](https://open-cmsis-pack.github.io/Open-CMSIS-Pack-Spec/main/html/pdsc_conditions_pg.html#element_condition)                                  | Reference to the condition ID of the software pack that triggered inclusion of this file.

<!-- markdownlint-restore -->

## `constructed-files:`

A list of files that are generated by the RTE management of the `csolution` tool.

`constructed-files:`                                     | Content
:--------------------------------------------------------|:------------------------------------
`- file:`                                                | Name and path to the file.

### `messages:`

Messages that apply to this project context.

`messages:`                                              | Content
:--------------------------------------------------------|:------------------------------------
`- error:`                                               | Error message
`- warning:`                                             | Warning message
`- info:`                                                | Info message
